@@ -75,6 +75,8 @@ module Hledger.Data.Amount (
   maxprecisionwithpoint,
   setAmountPrecision,
   withPrecision,
+  setFullPrecision,
+  setMinimalPrecision,
   setAmountInternalPrecision,
   withInternalPrecision,
   setAmountDecimalPoint,
@@ -124,13 +126,12 @@ module Hledger.Data.Amount (
 ) where
 
 import Data.Char (isDigit)
-import Data.Decimal (roundTo)
+import Data.Decimal (roundTo, decimalPlaces, normalizeDecimal)
 import Data.Function (on)
 import Data.List
 import Data.Map (findWithDefault)
 import Data.Maybe
 import Data.Time.Calendar (Day)
-import Data.Ord (comparing)
 -- import Data.Text (Text)
 import qualified Data.Text as T
 import Safe (maximumDef)
@@ -166,7 +167,7 @@ instance Num Amount where
 
 -- | The empty simple amount.
 amount, nullamt :: Amount
-amount = Amount{acommodity="", aquantity=0, aprice=NoPrice, astyle=amountstyle, amultiplier=False}
+amount = Amount{acommodity="", aquantity=0, aprice=NoPrice, astyle=amountstyle, aismultiplier=False}
 nullamt = amount
 
 -- | A temporary value for parsed transactions which had no amount specified.
@@ -278,6 +279,23 @@ setAmountPrecision p a@Amount{astyle=s} = a{astyle=s{asprecision=p}}
 withPrecision :: Amount -> Int -> Amount
 withPrecision = flip setAmountPrecision
 
+-- | Increase an amount's display precision, if necessary, enough so
+-- that it will be shown exactly, with all significant decimal places
+-- (excluding trailing zeros).
+setFullPrecision :: Amount -> Amount
+setFullPrecision a = setAmountPrecision p a
+  where
+    p                = max displayprecision normalprecision
+    displayprecision = asprecision $ astyle a
+    normalprecision  = fromIntegral $ decimalPlaces $ normalizeDecimal $ aquantity a
+
+-- | Set an amount's display precision to just enough so that it will
+-- be shown exactly, with all significant decimal places.
+setMinimalPrecision :: Amount -> Amount
+setMinimalPrecision a = setAmountPrecision normalprecision a
+  where
+    normalprecision  = fromIntegral $ decimalPlaces $ normalizeDecimal $ aquantity a
+
 -- | Get a string representation of an amount for debugging,
 -- appropriate to the current debug level. 9 shows maximum detail.
 showAmountDebug :: Amount -> String
@@ -450,7 +468,7 @@ commodityValue j valuationdate c
   where
     dbg = dbg8 ("using market price for "++T.unpack c)
     applicableprices =
-      [p | p <- sortBy (comparing mpdate) $ jmarketprices j
+      [p | p <- sortOn mpdate $ jmarketprices j
       , mpcommodity p == c
       , mpdate p <= valuationdate
       ]
