@@ -1,6 +1,10 @@
-{-# LANGUAGE Rank2Types #-}
-{-# LANGUAGE StandaloneDeriving, OverloadedStrings #-}
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE LambdaCase #-}
+{-# LANGUAGE NamedFieldPuns #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE Rank2Types #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE StandaloneDeriving #-}
 
 {-|
 
@@ -67,28 +71,31 @@ module Hledger.Data.Journal (
   journalCheckBalanceAssertions,
   journalNumberAndTieTransactions,
   journalUntieTransactions,
+  journalModifyTransactions,
   -- * Tests
   samplejournal,
   tests_Journal,
 )
 where
 import Control.Applicative (Const(..))
-import Control.Arrow
 import Control.Monad
 import Control.Monad.Except
-import qualified Control.Monad.Reader as R
+import Control.Monad.Extra
+import Control.Monad.Reader as R
 import Control.Monad.ST
 import Data.Array.ST
+import Data.Function ((&))
 import Data.Functor.Identity (Identity(..))
-import qualified Data.HashTable.ST.Cuckoo as HT
+import qualified Data.HashTable.ST.Cuckoo as H
 import Data.List
 import Data.List.Extra (groupSort)
+import qualified Data.Map as M
 import Data.Maybe
 #if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid
 #endif
-import Data.Ord
 import qualified Data.Semigroup as Sem
+import qualified Data.Set as S
 import Data.Text (Text)
 import qualified Data.Text as T
 import Safe (headMay, headDef)
@@ -96,8 +103,6 @@ import Data.Time.Calendar
 import Data.Tree
 import System.Time (ClockTime(TOD))
 import Text.Printf
-import qualified Data.Map as M
-import qualified Data.Set as S
 
 import Hledger.Utils 
 import Hledger.Data.Types
@@ -105,6 +110,7 @@ import Hledger.Data.AccountName
 import Hledger.Data.Amount
 import Hledger.Data.Dates
 import Hledger.Data.Transaction
+import Hledger.Data.TransactionModifier
 import Hledger.Data.Posting
 import Hledger.Query
 
@@ -262,7 +268,7 @@ journalAccountNamesImplied = expandAccountNames . journalAccountNamesUsed
 
 -- | Sorted unique account names declared by account directives in this journal.
 journalAccountNamesDeclared :: Journal -> [AccountName]
-journalAccountNamesDeclared = nub . sort . jdeclaredaccounts
+journalAccountNamesDeclared = nub . sort . map fst . jdeclaredaccounts
 
 -- | Sorted unique account names declared by account directives or posted to
 -- by transactions in this journal.
@@ -555,261 +561,328 @@ journalTieTransactions j@Journal{jtxns=ts} = j{jtxns=map txnTieKnot ts}
 journalUntieTransactions :: Transaction -> Transaction
 journalUntieTransactions t@Transaction{tpostings=ps} = t{tpostings=map (\p -> p{ptransaction=Nothing}) ps}
 
--- | Check any balance assertions in the journal and return an error
--- message if any of them fail.
-journalCheckBalanceAssertions :: Journal -> Either String Journal
-journalCheckBalanceAssertions j =
-  runST $ journalBalanceTransactionsST 
-    True 
-    j 
-    (return ())
-    (\_ _ -> return ()) 
-    (const $ return j)
+-- | Apply any transaction modifier rules in the journal 
+-- (adding automated postings to transactions, eg).
+journalModifyTransactions :: Journal -> Journal
+journalModifyTransactions j = j{ jtxns = modifyTransactions (jtxnmodifiers j) (jtxns j) }
 
--- | Check a posting's balance assertion and return an error if it
--- fails.
-checkBalanceAssertion :: Posting -> MixedAmount -> Either String ()
-checkBalanceAssertion p@Posting{ pbalanceassertion = Just ass } bal =
-  foldl' fold (Right ()) amts
-    where fold (Right _) cass = checkBalanceAssertionCommodity p cass bal
-          fold err _ = err
-          amt = baamount ass
-          amts = amt : if baexact ass
-            then map (\a -> a{ aquantity = 0 }) $ amounts $ filterMixedAmount (\a -> acommodity a /= assertedcomm) bal
-            else []
-          assertedcomm = acommodity amt
-checkBalanceAssertion _ _ = Right ()
+-- | Check any balance assertions in the journal and return an error message
+-- if any of them fail (or if the transaction balancing they require fails).
+journalCheckBalanceAssertions :: Journal -> Maybe String
+journalCheckBalanceAssertions = either Just (const Nothing) . journalBalanceTransactions True
 
-checkBalanceAssertionCommodity :: Posting -> Amount -> MixedAmount -> Either String ()
-checkBalanceAssertionCommodity p amt bal
-  | isReallyZeroAmount diff = Right ()
-  | True    = Left err
-    where assertedcomm = acommodity amt
-          actualbal = fromMaybe nullamt $ find ((== assertedcomm) . acommodity) (amounts bal)
-          diff = amt - actualbal
-          diffplus | isNegativeAmount diff == False = "+"
-                   | otherwise = ""
-          err = printf (unlines
-                        [ "balance assertion error%s",
-                          "after posting:",
-                          "%s",
-                          "balance assertion details:",
-                          "date:       %s",
-                          "account:    %s",
-                          "commodity:  %s",
-                          "calculated: %s",
-                          "asserted:   %s (difference: %s)"
-                        ])
-            (case ptransaction p of
-               Nothing -> ":" -- shouldn't happen
-               Just t ->  printf " in %s:\nin transaction:\n%s"
-                          (showGenericSourcePos pos) (chomp $ showTransaction t) :: String
-                            where pos = baposition $ fromJust $ pbalanceassertion p)
-            (showPostingLine p)
-            (showDate $ postingDate p)
-            (T.unpack $ paccount p) -- XXX pack
-            assertedcomm
-            (showAmount actualbal)
-            (showAmount amt)
-            (diffplus ++ showAmount diff)
+-- "Transaction balancing" - inferring missing amounts and checking transaction balancedness and balance assertions
 
--- | Fill in any missing amounts and check that all journal transactions
--- balance, or return an error message. This is done after parsing all
--- amounts and applying canonical commodity styles, since balancing
--- depends on display precision. Reports only the first error encountered.
-journalBalanceTransactions :: Bool -> Journal -> Either String Journal
-journalBalanceTransactions assrt j =
-  runST $ journalBalanceTransactionsST 
-    assrt -- check balance assertions also ?
-    (journalNumberTransactions j) -- journal to process
-    (newArray_ (1, genericLength $ jtxns j) :: forall s. ST s (STArray s Integer Transaction)) -- initialise state
-    (\arr tx -> writeArray arr (tindex tx) tx)    -- update state
-    (fmap (\txns -> j{ jtxns = txns}) . getElems) -- summarise state
+-- | Monad used for statefully balancing/amount-inferring/assertion-checking 
+-- a sequence of transactions.
+-- Perhaps can be simplified, or would a different ordering of layers make sense ?
+-- If you see a way, let us know.
+type Balancing s = ReaderT (BalancingState s) (ExceptT String (ST s))
 
--- | Helper used by 'journalBalanceTransactions' and 'journalCheckBalanceAssertions'.
-journalBalanceTransactionsST ::
-  Bool
-  -> Journal
-  -> ST s txns                        -- ^ initialise state
-  -> (txns -> Transaction -> ST s ()) -- ^ update state
-  -> (txns -> ST s a)                 -- ^ summarise state
-  -> ST s (Either String a)
-journalBalanceTransactionsST assrt j createStore storeIn extract =
-  runExceptT $ do
-    bals <- lift $ HT.newSized size
-    txStore <- lift $ createStore
-    let env = Env bals 
-                  (storeIn txStore) 
-                  assrt
-                  (Just $ journalCommodityStyles j)
-                  (getModifierAccountNames j)
-    flip R.runReaderT env $ do
-      dated <- fmap snd . sortBy (comparing fst) . concat
-                <$> mapM' discriminateByDate (jtxns j)
-      mapM' checkInferAndRegisterAmounts dated
-    lift $ extract txStore
-    where 
-      size = genericLength $ journalPostings j
+-- | The state used while balancing a sequence of transactions.
+data BalancingState s = BalancingState {
+   -- read only
+   bsStyles       :: Maybe (M.Map CommoditySymbol AmountStyle)  -- ^ commodity display styles
+  ,bsUnassignable :: S.Set AccountName                          -- ^ accounts in which balance assignments may not be used
+  ,bsAssrt        :: Bool                                       -- ^ whether to check balance assertions
+   -- mutable
+  ,bsBalances     :: H.HashTable s AccountName MixedAmount      -- ^ running account balances, initially empty
+  ,bsTransactions :: STArray s Integer Transaction              -- ^ the transactions being balanced
+  }
 
+-- | Access the current balancing state, and possibly modify the mutable bits,
+-- lifting through the Except and Reader layers into the Balancing monad.
+withB :: (BalancingState s -> ST s a) -> Balancing s a
+withB f = ask >>= lift . lift . f
 
--- | Collect account names in account modifiers into a set
-getModifierAccountNames :: Journal -> S.Set AccountName
-getModifierAccountNames j = S.fromList $
-                            map paccount $
-                            concatMap tmpostingrules $
-                            jtxnmodifiers j
+-- | Get an account's running balance so far. 
+getAmountB :: AccountName -> Balancing s MixedAmount
+getAmountB acc = withB $ \BalancingState{bsBalances} -> do 
+  fromMaybe 0 <$> H.lookup bsBalances acc
 
--- | Monad transformer stack with a reference to a mutable hashtable
--- of current account balances and a mutable array of finished
--- transactions in original parsing order.
-type CurrentBalancesModifier s = R.ReaderT (Env s) (ExceptT String (ST s))
-
--- | Environment for 'CurrentBalancesModifier'
-data Env s = Env { eBalances     :: HT.HashTable s AccountName MixedAmount
-                 , eStoreTx      :: Transaction -> ST s ()
-                 , eAssrt        :: Bool
-                 , eStyles       :: Maybe (M.Map CommoditySymbol AmountStyle)
-                 , eUnassignable :: S.Set AccountName
-                 }
-
--- | This converts a transaction into a list of transactions or
--- postings whose dates have to be considered when checking 
--- balance assertions and handled by 'checkInferAndRegisterAmounts'.
---
--- Transaction without balance assignments can be balanced and stored
--- immediately and their (possibly) dated postings are returned.
---
--- Transaction with balance assignments are only supported if no
--- posting has a 'pdate' value. Supported transactions will be
--- returned unchanged and balanced and stored later in 'checkInferAndRegisterAmounts'.
-discriminateByDate :: Transaction
-  -> CurrentBalancesModifier s [(Day, Either Posting Transaction)]
-discriminateByDate tx
-  | null (assignmentPostings tx) = do
-      styles <- R.reader $ eStyles
-      balanced <- lift $ ExceptT $ return $ balanceTransaction styles tx
-      storeTransaction balanced
-      return $ 
-        fmap (postingDate &&& (Left . removePrices)) $ tpostings $ balanced
-  | True                         = do
-      when (any (isJust . pdate) $ tpostings tx) $
-        throwError $ unlines $
-        ["postings may not have both a custom date and a balance assignment."
-        ,"Write the posting amount explicitly, or remove the posting date:\n"
-        , showTransaction tx]
-      return 
-        [(tdate tx, Right $ tx { tpostings = removePrices <$> tpostings tx })]
-
--- | Throw an error if a posting is in the unassignable set.
-checkUnassignablePosting :: Posting -> CurrentBalancesModifier s ()
-checkUnassignablePosting p = do
-  unassignable <- R.asks eUnassignable
-  if (isAssignment p && paccount p `S.member` unassignable)
-    then throwError $ unlines $
-         [ "cannot assign amount to account "
-         , ""
-         , "    " ++ (T.unpack $ paccount p)
-         , ""
-         , "because it is also included in transaction modifiers."
-         ]
-    else return ()
-
-
--- | This function takes an object describing changes to
--- account balances on a single day - either a single posting 
--- (from an already balanced transaction without assignments)
--- or a whole transaction with assignments (which is required to 
--- have no posting with pdate set).
---
--- For a single posting, there is not much to do. Only add its amount
--- to its account and check the assertion, if there is one. This
--- functionality is provided by 'addAmountAndCheckBalance'.
---
--- For a whole transaction, it loops over all postings, and performs
--- 'addAmountAndCheckBalance', if there is an amount. If there is no
--- amount, the amount is inferred by the assertion or left empty if
--- there is no assertion. Then, the transaction is balanced, the
--- inferred amount added to the balance (all in 'balanceTransactionUpdate') 
--- and the resulting transaction with no missing amounts is stored 
--- in the array, for later retrieval.
---
--- Again in short:
---
--- 'Left Posting': Check the balance assertion and update the
---  account balance. If the amount is empty do nothing.  this can be
---  the case e.g. for virtual postings
---
--- 'Right Transaction': Loop over all postings, infer their amounts
--- and then balance and store the transaction.
-checkInferAndRegisterAmounts :: Either Posting Transaction
-                             -> CurrentBalancesModifier s ()
-checkInferAndRegisterAmounts (Left p) = do
-  checkUnassignablePosting p
-  void $ addAmountAndCheckBalance return p
-checkInferAndRegisterAmounts (Right oldTx) = do
-  let ps = tpostings oldTx
-  mapM_ checkUnassignablePosting ps
-  styles <- R.reader $ eStyles
-  newPostings <- forM ps $ addAmountAndCheckBalance inferFromAssignment
-  storeTransaction =<< balanceTransactionUpdate
-    (fmap void . addToBalance) styles oldTx { tpostings = newPostings }
-  where
-    inferFromAssignment :: Posting -> CurrentBalancesModifier s Posting
-    inferFromAssignment p = do
-      let acc = paccount p
-      case pbalanceassertion p of
-        Just ba | baexact ba -> do
-          diff <- setMixedBalance acc $ Mixed [baamount ba]
-          fullPosting diff p
-        Just ba | otherwise -> do
-          old <- liftModifier $ \Env{ eBalances = bals } -> HT.lookup bals acc
-          let amt = baamount ba
-              assertedcomm = acommodity amt
-          diff <- setMixedBalance acc $
-            Mixed [amt] + filterMixedAmount (\a -> acommodity a /= assertedcomm) (fromMaybe nullmixedamt old)
-          fullPosting diff p
-        Nothing -> return p
-    fullPosting amt p = return p
-      { pamount = amt
-      , porigin = Just $ originalPosting p
-      }
-
--- | Adds a posting's amount to the posting's account balance and
--- checks a possible balance assertion. Or if there is no amount,
--- runs the supplied fallback action.
-addAmountAndCheckBalance :: 
-     (Posting -> CurrentBalancesModifier s Posting) -- ^ action if posting has no amount
-  -> Posting
-  -> CurrentBalancesModifier s Posting
-addAmountAndCheckBalance _ p | hasAmount p = do
-  newAmt <- addToBalance (paccount p) $ pamount p
-  assrt <- R.reader eAssrt
-  lift $ when assrt $ ExceptT $ return $ checkBalanceAssertion p newAmt
-  return p
-addAmountAndCheckBalance fallback p = fallback p
-
--- | Sets all commodities comprising an account's balance to the given
--- amounts and returns the difference from the previous balance.
-setMixedBalance :: AccountName -> MixedAmount -> CurrentBalancesModifier s MixedAmount
-setMixedBalance acc amt = liftModifier $ \Env{ eBalances = bals } -> do
-  old <- HT.lookup bals acc
-  HT.insert bals acc amt
-  return $ maybe amt (amt -) old
-
--- | Adds an amount to an account's balance and returns the resulting balance.
-addToBalance :: AccountName -> MixedAmount -> CurrentBalancesModifier s MixedAmount
-addToBalance acc amt = liftModifier $ \Env{ eBalances = bals } -> do
-  new <- maybe amt (+ amt) <$> HT.lookup bals acc
-  HT.insert bals acc new
+-- | Add an amount to an account's running balance, and return the new running balance.
+addAmountB :: AccountName -> MixedAmount -> Balancing s MixedAmount
+addAmountB acc amt = withB $ \BalancingState{bsBalances} -> do
+  old <- fromMaybe 0 <$> H.lookup bsBalances acc
+  let new = old + amt
+  H.insert bsBalances acc new
   return new
 
--- | Stores a transaction in the transaction array in original parsing order.
-storeTransaction :: Transaction -> CurrentBalancesModifier s ()
-storeTransaction tx = liftModifier $ ($tx) . eStoreTx
+-- | Set an account's running balance to this amount, and return the difference from the old. 
+setAmountB :: AccountName -> MixedAmount -> Balancing s MixedAmount
+setAmountB acc amt = withB $ \BalancingState{bsBalances} -> do
+  old <- fromMaybe 0 <$> H.lookup bsBalances acc
+  H.insert bsBalances acc amt
+  return $ amt - old
 
--- | Helper function.
-liftModifier :: (Env s -> ST s a) -> CurrentBalancesModifier s a
-liftModifier f = R.ask >>= lift . lift . f
+-- | Update (overwrite) this transaction with a new one.
+storeTransactionB :: Transaction -> Balancing s ()
+storeTransactionB t = withB $ \BalancingState{bsTransactions}  ->
+  void $ writeArray bsTransactions (tindex t) t
+
+-- | Infer any missing amounts (to satisfy balance assignments and
+-- to balance transactions) and check that all transactions balance 
+-- and (optional) all balance assertions pass. Or return an error message
+-- (just the first error encountered).
+--
+-- Assumes journalInferCommodityStyles has been called, since those affect transaction balancing.
+--
+-- This does multiple things because amount inferring, balance assignments, 
+-- balance assertions and posting dates are interdependent.
+-- 
+-- This can be simplified further. Overview as of 20190219:
+-- @
+-- ****** parseAndFinaliseJournal['] (Cli/Utils.hs), journalAddForecast (Common.hs), budgetJournal (BudgetReport.hs), tests (BalanceReport.hs)
+-- ******* journalBalanceTransactions
+-- ******** runST
+-- ********* runExceptT
+-- ********** balanceTransaction (Transaction.hs)
+-- *********** balanceTransactionHelper
+-- ********** runReaderT
+-- *********** balanceTransactionAndCheckAssertionsB
+-- ************ addAmountAndCheckAssertionB
+-- ************ addOrAssignAmountAndCheckAssertionB
+-- ************ balanceTransactionHelper (Transaction.hs)
+-- ****** uiCheckBalanceAssertions d ui@UIState{aopts=UIOpts{cliopts_=copts}, ajournal=j} (ErrorScreen.hs)
+-- ******* journalCheckBalanceAssertions
+-- ******** journalBalanceTransactions
+-- ****** transactionWizard, postingsBalanced (Add.hs), tests (Transaction.hs)
+-- ******* balanceTransaction (Transaction.hs)  XXX hledger add won't allow balance assignments + missing amount ?
+-- @
+journalBalanceTransactions :: Bool -> Journal -> Either String Journal
+journalBalanceTransactions assrt j' =
+  let
+    -- ensure transactions are numbered, so we can store them by number 
+    j@Journal{jtxns=ts} = journalNumberTransactions j'
+    -- display precisions used in balanced checking
+    styles = Just $ journalCommodityStyles j
+    -- balance assignments will not be allowed on these
+    txnmodifieraccts = S.fromList $ map paccount $ concatMap tmpostingrules $ jtxnmodifiers j 
+  in 
+    runST $ do 
+      -- We'll update a mutable array of transactions as we balance them,
+      -- not strictly necessary but avoids a sort at the end I think.
+      balancedtxns <- newListArray (1, genericLength ts) ts
+
+      -- Infer missing posting amounts, check transactions are balanced, 
+      -- and check balance assertions. This is done in two passes:
+      runExceptT $ do
+
+        -- 1. Step through the transactions, balancing the ones which don't have balance assignments
+        -- and leaving the others for later. The balanced ones are split into their postings.
+        -- The postings and not-yet-balanced transactions remain in the same relative order.
+        psandts :: [Either Posting Transaction] <- fmap concat $ forM ts $ \case
+          t | null $ assignmentPostings t -> case balanceTransaction styles t of
+              Left  e  -> throwError e 
+              Right t' -> do
+                lift $ writeArray balancedtxns (tindex t') t'
+                return $ map Left $ tpostings t'
+          t -> return [Right t]
+
+        -- 2. Sort these items by date, preserving the order of same-day items,
+        -- and step through them while keeping running account balances, 
+        runningbals <- lift $ H.newSized (length $ journalAccountNamesUsed j)
+        flip runReaderT (BalancingState styles txnmodifieraccts assrt runningbals balancedtxns) $ do
+          -- performing balance assignments in, and balancing, the remaining transactions,
+          -- and checking balance assertions as each posting is processed.
+          void $ mapM' balanceTransactionAndCheckAssertionsB $ sortOn (either postingDate tdate) psandts
+
+        ts' <- lift $ getElems balancedtxns
+        return j{jtxns=ts'} 
+
+-- | This function is called statefully on each of a date-ordered sequence of 
+-- 1. fully explicit postings from already-balanced transactions and 
+-- 2. not-yet-balanced transactions containing balance assignments.
+-- It executes balance assignments and finishes balancing the transactions, 
+-- and checks balance assertions on each posting as it goes.
+-- An error will be thrown if a transaction can't be balanced 
+-- or if an illegal balance assignment is found (cf checkIllegalBalanceAssignment).
+-- Transaction prices are removed, which helps eg balance-assertions.test: 15. Mix different commodities and assignments.
+-- This stores the balanced transactions in case 2 but not in case 1.  
+balanceTransactionAndCheckAssertionsB :: Either Posting Transaction -> Balancing s ()
+
+balanceTransactionAndCheckAssertionsB (Left p@Posting{}) =
+  -- update the account's running balance and check the balance assertion if any
+  void $ addAmountAndCheckAssertionB $ removePrices p
+
+balanceTransactionAndCheckAssertionsB (Right t@Transaction{tpostings=ps}) = do
+  -- make sure we can handle the balance assignments
+  mapM_ checkIllegalBalanceAssignmentB ps
+  -- for each posting, infer its amount from the balance assignment if applicable, 
+  -- update the account's running balance and check the balance assertion if any
+  ps' <- forM ps $ \p -> pure (removePrices p) >>= addOrAssignAmountAndCheckAssertionB
+  -- infer any remaining missing amounts, and make sure the transaction is now fully balanced 
+  styles <- R.reader bsStyles
+  case balanceTransactionHelper styles t{tpostings=ps'} of
+    Left err -> throwError err 
+    Right (t', inferredacctsandamts) -> do
+      -- for each amount just inferred, update the running balance 
+      mapM_ (uncurry addAmountB) inferredacctsandamts
+      -- and save the balanced transaction.
+      storeTransactionB t' 
+
+-- | If this posting has an explicit amount, add it to the account's running balance.
+-- If it has a missing amount and a balance assignment, infer the amount from, and 
+-- reset the running balance to, the assigned balance.
+-- If it has a missing amount and no balance assignment, leave it for later.
+-- Then test the balance assertion if any.
+addOrAssignAmountAndCheckAssertionB :: Posting -> Balancing s Posting
+addOrAssignAmountAndCheckAssertionB p@Posting{paccount=acc, pamount=amt, pbalanceassertion=mba}
+  | hasAmount p = do
+      newbal <- addAmountB acc amt 
+      whenM (R.reader bsAssrt) $ checkBalanceAssertionB p newbal
+      return p
+  | Just BalanceAssertion{baamount,batotal} <- mba = do
+      (diff,newbal) <- case batotal of
+        True  -> do
+          -- a total balance assignment
+          let newbal = Mixed [baamount]
+          diff <- setAmountB acc newbal
+          return (diff,newbal)
+        False -> do
+          -- a partial balance assignment
+          oldbalothercommodities <- filterMixedAmount ((acommodity baamount /=) . acommodity) <$> getAmountB acc
+          let assignedbalthiscommodity = Mixed [baamount] 
+              newbal = oldbalothercommodities + assignedbalthiscommodity   
+          diff <- setAmountB acc newbal
+          return (diff,newbal)
+      let p' = p{pamount=diff, poriginal=Just $ originalPosting p}
+      whenM (R.reader bsAssrt) $ checkBalanceAssertionB p' newbal
+      return p'
+  -- no amount, no balance assertion (GHC 7 doesn't like Nothing <- mba here)
+  | otherwise = return p
+
+-- | Add the posting's amount to its account's running balance, and
+-- optionally check the posting's balance assertion if any.
+-- The posting is expected to have an explicit amount (otherwise this does nothing).
+-- Adding and checking balance assertions are tightly paired because we
+-- need to see the balance as it stands after each individual posting. 
+addAmountAndCheckAssertionB :: Posting -> Balancing s Posting
+addAmountAndCheckAssertionB p | hasAmount p = do
+  newbal <- addAmountB (paccount p) (pamount p)
+  whenM (R.reader bsAssrt) $ checkBalanceAssertionB p newbal
+  return p
+addAmountAndCheckAssertionB p = return p
+
+-- | Check a posting's balance assertion against the given actual balance, and
+-- return an error if the assertion is not satisfied.
+-- If the assertion is partial, unasserted commodities in the actual balance
+-- are ignored; if it is total, they will cause the assertion to fail.
+checkBalanceAssertionB :: Posting -> MixedAmount -> Balancing s ()
+checkBalanceAssertionB p@Posting{pbalanceassertion=Just (BalanceAssertion{baamount,batotal})} actualbal =
+  forM_ assertedamts $ \amt -> checkBalanceAssertionOneCommodityB p amt actualbal
+  where
+    assertedamts = baamount : otheramts
+      where
+        assertedcomm = acommodity baamount
+        otheramts | batotal   = map (\a -> a{aquantity=0}) $ amounts $ filterMixedAmount ((/=assertedcomm).acommodity) actualbal
+                  | otherwise = []
+checkBalanceAssertionB _ _ = return ()
+
+-- | Does this (single commodity) expected balance match the amount of that
+-- commodity in the given (multicommodity) actual balance ? If not, returns a
+-- balance assertion failure message based on the provided posting.  To match,
+-- the amounts must be exactly equal (display precision is ignored here).
+-- If the assertion is inclusive, the expected amount is compared with the account's
+-- subaccount-inclusive balance; otherwise, with the subaccount-exclusive balance.
+checkBalanceAssertionOneCommodityB :: Posting -> Amount -> MixedAmount -> Balancing s ()
+checkBalanceAssertionOneCommodityB p@Posting{paccount=assertedacct} assertedamt actualbal = do
+  let isinclusive = maybe False bainclusive $ pbalanceassertion p
+  actualbal' <- 
+    if isinclusive 
+    then 
+      -- sum the running balances of this account and any of its subaccounts seen so far 
+      withB $ \BalancingState{bsBalances} -> 
+        H.foldM 
+          (\ibal (acc, amt) -> return $ ibal + 
+            if assertedacct==acc || assertedacct `isAccountNamePrefixOf` acc then amt else 0)
+          0 
+          bsBalances
+    else return actualbal  
+  let
+    assertedcomm    = acommodity assertedamt
+    actualbalincomm = headDef 0 $ amounts $ filterMixedAmountByCommodity assertedcomm $ actualbal'
+    pass =
+      aquantity
+        -- traceWith (("asserted:"++).showAmountDebug)
+        assertedamt ==
+      aquantity
+        -- traceWith (("actual:"++).showAmountDebug)
+        actualbalincomm
+
+    errmsg = printf (unlines
+                  [ "balance assertion: %s",
+                    "\nassertion details:",
+                    "date:       %s",
+                    "account:    %s%s",
+                    "commodity:  %s",
+                    -- "display precision:  %d",
+                    "calculated: %s", -- (at display precision: %s)",
+                    "asserted:   %s", -- (at display precision: %s)",
+                    "difference: %s"
+                  ])
+      (case ptransaction p of
+         Nothing -> "?" -- shouldn't happen
+         Just t ->  printf "%s\ntransaction:\n%s"
+                      (showGenericSourcePos pos)
+                      (chomp $ showTransaction t)
+                      :: String
+                      where
+                        pos = baposition $ fromJust $ pbalanceassertion p
+      )
+      (showDate $ postingDate p)
+      (T.unpack $ paccount p) -- XXX pack
+      (if isinclusive then " (and subs)" else "" :: String)
+      assertedcomm
+      -- (asprecision $ astyle actualbalincommodity)  -- should be the standard display precision I think
+      (show $ aquantity actualbalincomm)
+      -- (showAmount actualbalincommodity)
+      (show $ aquantity assertedamt)
+      -- (showAmount assertedamt)
+      (show $ aquantity assertedamt - aquantity actualbalincomm)
+
+  when (not pass) $ throwError errmsg
+
+-- | Throw an error if this posting is trying to do an illegal balance assignment.
+checkIllegalBalanceAssignmentB :: Posting -> Balancing s ()
+checkIllegalBalanceAssignmentB p = do 
+  checkBalanceAssignmentPostingDateB p
+  checkBalanceAssignmentUnassignableAccountB p
+  
+-- XXX these should show position. annotateErrorWithTransaction t ?
+
+-- | Throw an error if this posting is trying to do a balance assignment and
+-- has a custom posting date (which makes amount inference too hard/impossible).
+checkBalanceAssignmentPostingDateB :: Posting -> Balancing s ()
+checkBalanceAssignmentPostingDateB p =
+  when (hasBalanceAssignment p && isJust (pdate p)) $ 
+    throwError $ unlines $
+      ["postings which are balance assignments may not have a custom date."
+      ,"Please write the posting amount explicitly, or remove the posting date:"
+      ,""
+      ,maybe (unlines $ showPostingLines p) showTransaction $ ptransaction p
+      ]
+
+-- | Throw an error if this posting is trying to do a balance assignment and
+-- the account does not allow balance assignments (eg because it is referenced
+-- by a transaction modifier, which might generate additional postings to it).
+checkBalanceAssignmentUnassignableAccountB :: Posting -> Balancing s ()
+checkBalanceAssignmentUnassignableAccountB p = do
+  unassignable <- R.asks bsUnassignable
+  when (hasBalanceAssignment p && paccount p `S.member` unassignable) $
+    throwError $ unlines $
+      ["balance assignments cannot be used with accounts which are"
+      ,"posted to by transaction modifier rules (auto postings)."
+      ,"Please write the posting amount explicitly, or remove the rule."
+      ,""
+      ,"account: "++T.unpack (paccount p)
+      ,""
+      ,"transaction:"
+      ,""
+      ,maybe (unlines $ showPostingLines p) showTransaction $ ptransaction p
+      ]
+
+--
 
 -- | Choose and apply a consistent display format to the posting
 -- amounts in each commodity. Each commodity's format is specified by
@@ -856,13 +929,12 @@ commodityStylesFromAmounts amts = M.fromList commstyles
 -- That is: the style of the first, and the maximum precision of all.
 canonicalStyleFrom :: [AmountStyle] -> AmountStyle
 canonicalStyleFrom [] = amountstyle
-canonicalStyleFrom ss@(first:_) =
-  first{asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps}
+canonicalStyleFrom ss@(first:_) = first {asprecision = prec, asdecimalpoint = mdec, asdigitgroups = mgrps}
   where
-    mgrps = maybe Nothing Just $ headMay $ catMaybes $ map asdigitgroups ss
+    mgrps = headMay $ mapMaybe asdigitgroups ss
     -- precision is maximum of all precisions
     prec = maximumStrict $ map asprecision ss
-    mdec  = Just $ headDef '.' $ catMaybes $ map asdecimalpoint ss
+    mdec = Just $ headDef '.' $ mapMaybe asdecimalpoint ss
     -- precision is that of first amount with a decimal point
     -- (mdec, prec) =
     --   case filter (isJust . asdecimalpoint) ss of
@@ -965,7 +1037,7 @@ journalDateSpan secondary j
       latest   = maximumStrict dates
       dates    = pdates ++ tdates
       tdates   = map (if secondary then transactionDate2 else tdate) ts
-      pdates   = concatMap (catMaybes . map (if secondary then (Just . postingDate2) else pdate) . tpostings) ts
+      pdates   = concatMap (mapMaybe (if secondary then (Just . postingDate2) else pdate) . tpostings) ts
       ts       = jtxns j
 
 -- | Apply the pivot transformation to all postings in a journal,
@@ -981,7 +1053,7 @@ transactionPivot fieldortagname t = t{tpostings = map (postingPivot fieldortagna
 -- | Replace this posting's account name with the value
 -- of the given field or tag, if any, otherwise the empty string.
 postingPivot :: Text -> Posting -> Posting         
-postingPivot fieldortagname p = p{paccount = pivotedacct, porigin = Just $ originalPosting p}
+postingPivot fieldortagname p = p{paccount = pivotedacct, poriginal = Just $ originalPosting p}
   where
     pivotedacct
       | Just t <- ptransaction p, fieldortagname == "code"        = tcode t  
@@ -1063,7 +1135,7 @@ Right samplejournal = journalBalanceTransactions False $
                  ["assets:bank:checking" `post` usd 1
                  ,"income:salary" `post` missingamt
                  ],
-             tpreceding_comment_lines=""
+             tprecedingcomment=""
            }
           ,
            txnTieKnot $ Transaction {
@@ -1080,7 +1152,7 @@ Right samplejournal = journalBalanceTransactions False $
                  ["assets:bank:checking" `post` usd 1
                  ,"income:gifts" `post` missingamt
                  ],
-             tpreceding_comment_lines=""
+             tprecedingcomment=""
            }
           ,
            txnTieKnot $ Transaction {
@@ -1097,7 +1169,7 @@ Right samplejournal = journalBalanceTransactions False $
                  ["assets:bank:saving" `post` usd 1
                  ,"assets:bank:checking" `post` usd (-1)
                  ],
-             tpreceding_comment_lines=""
+             tprecedingcomment=""
            }
           ,
            txnTieKnot $ Transaction {
@@ -1114,7 +1186,7 @@ Right samplejournal = journalBalanceTransactions False $
                        ,"expenses:supplies" `post` usd 1
                        ,"assets:cash" `post` missingamt
                        ],
-             tpreceding_comment_lines=""
+             tprecedingcomment=""
            }
           ,
            txnTieKnot $ Transaction {
@@ -1130,7 +1202,7 @@ Right samplejournal = journalBalanceTransactions False $
              tpostings=["assets:bank:checking" `post` usd 1
                        ,"liabilities:debts" `post` usd (-1)
                        ],
-             tpreceding_comment_lines=""
+             tprecedingcomment=""
            }
           ,
            txnTieKnot $ Transaction {
@@ -1146,7 +1218,7 @@ Right samplejournal = journalBalanceTransactions False $
              tpostings=["liabilities:debts" `post` usd 1
                        ,"assets:bank:checking" `post` usd (-1)
                        ],
-             tpreceding_comment_lines=""
+             tprecedingcomment=""
            }
           ]
          }
@@ -1179,4 +1251,59 @@ tests_Journal = tests "Journal" [
       ,test "expenses"    $ expectEq (namesfrom journalExpenseAccountQuery)   ["expenses","expenses:food","expenses:supplies"]
     ]
 
+  ,tests "journalBalanceTransactions" [
+
+     test "balance-assignment" $ do
+      let ej = journalBalanceTransactions True $
+            --2019/01/01
+            --  (a)            = 1
+            nulljournal{ jtxns = [
+              transaction "2019/01/01" [ vpost' "a" missingamt (balassert (num 1)) ]
+            ]}
+      expectRight ej
+      let Right j = ej
+      (jtxns j & head & tpostings & head & pamount) `is` Mixed [num 1]
+
+    ,test "same-day-1" $ do
+      expectRight $ journalBalanceTransactions True $
+            --2019/01/01
+            --  (a)            = 1
+            --2019/01/01
+            --  (a)          1 = 2
+            nulljournal{ jtxns = [
+               transaction "2019/01/01" [ vpost' "a" missingamt (balassert (num 1)) ]
+              ,transaction "2019/01/01" [ vpost' "a" (num 1)    (balassert (num 2)) ]
+            ]}
+
+    ,test "same-day-2" $ do
+      expectRight $ journalBalanceTransactions True $
+            --2019/01/01
+            --    (a)                  2 = 2
+            --2019/01/01
+            --    b                    1
+            --    a
+            --2019/01/01
+            --    a                    0 = 1
+            nulljournal{ jtxns = [
+               transaction "2019/01/01" [ vpost' "a" (num 2)    (balassert (num 2)) ]
+              ,transaction "2019/01/01" [
+                 post' "b" (num 1)     Nothing  
+                ,post' "a"  missingamt Nothing  
+              ]
+              ,transaction "2019/01/01" [ post' "a" (num 0)     (balassert (num 1)) ]
+            ]}
+
+    ,test "out-of-order" $ do
+      expectRight $ journalBalanceTransactions True $
+            --2019/1/2
+            --  (a)    1 = 2
+            --2019/1/1
+            --  (a)    1 = 1
+            nulljournal{ jtxns = [
+               transaction "2019/01/02" [ vpost' "a" (num 1)    (balassert (num 2)) ]
+              ,transaction "2019/01/01" [ vpost' "a" (num 1)    (balassert (num 1)) ]
+            ]}
+
+    ]
+
   ]
