@@ -107,12 +107,13 @@ ledgerCommodities = M.keys . jinferredcommodities . ljournal
 
 -- tests
 
-tests_Ledger = tests "Ledger" [
-
-  tests "ledgerFromJournal" [
-     (length $ ledgerPostings $ ledgerFromJournal Any nulljournal) `is` 0
-    ,(length $ ledgerPostings $ ledgerFromJournal Any samplejournal) `is` 13
-    ,(length $ ledgerPostings $ ledgerFromJournal (Depth 2) samplejournal) `is` 7
-  ]
-
- ]
+tests_Ledger =
+  tests
+    "Ledger"
+    [ tests
+        "ledgerFromJournal"
+        [ length (ledgerPostings $ ledgerFromJournal Any nulljournal) `is` 0
+        , length (ledgerPostings $ ledgerFromJournal Any samplejournal) `is` 13
+        , length (ledgerPostings $ ledgerFromJournal (Depth 2) samplejournal) `is` 7
+        ]
+    ]
