@@ -26,6 +26,7 @@ import Control.DeepSeq (NFData)
 import Data.Data
 import Data.Decimal
 import Data.Default
+import Data.Functor (($>))
 import Data.List (intercalate)
 import Text.Blaze (ToMarkup(..))
 --XXX https://hackage.haskell.org/package/containers/docs/Data-Map.html 
@@ -201,12 +202,12 @@ data Commodity = Commodity {
 instance NFData Commodity
 
 data Amount = Amount {
-      acommodity  :: CommoditySymbol,
-      aquantity   :: Quantity,
-      aprice      :: Price,           -- ^ the (fixed) price for this amount, if any
+      acommodity  :: CommoditySymbol,   -- commodity symbol, or special value "AUTO"
+      aquantity   :: Quantity,          -- numeric quantity, or zero in case of "AUTO"
+      aismultiplier :: Bool,            -- ^ kludge: a flag marking this amount and posting as a multiplier
+                                        --   in a TMPostingRule. In a regular Posting, should always be false.
       astyle      :: AmountStyle,
-      amultiplier :: Bool             -- ^ kludge: a flag marking this amount and posting as a multiplier
-                                      --   in a TMPostingRule. In a regular Posting, should always be false.
+      aprice      :: Price            -- ^ the (fixed, transaction-specific) price for this amount, if any
     } deriving (Eq,Ord,Typeable,Data,Generic,Show)
 
 instance NFData Amount
@@ -237,12 +238,48 @@ instance Show Status where -- custom show.. bad idea.. don't do it..
   show Pending   = "!"
   show Cleared   = "*"
 
--- | The amount to compare an account's balance to, to verify that the history
--- leading to a given point is correct or to set the account to a known value.
+-- | A balance assertion is a declaration about an account's expected balance
+-- at a certain point (posting date and parse order). They provide additional
+-- error checking and readability to a journal file.
+--
+-- The 'BalanceAssertion' type is also used to represent balance assignments,
+-- which instruct hledger what an account's balance should become at a certain
+-- point.
+--
+-- Different kinds of balance assertions are discussed eg on #290.
+-- Variables include:
+--
+-- - which postings are to be summed (real/virtual; unmarked/pending/cleared; this account/this account including subs)
+--
+-- - which commodities within the balance are to be checked
+--
+-- - whether to do a partial or a total check (disallowing other commodities)
+--
+-- I suspect we want:
+--
+-- 1. partial, subaccount-exclusive, Ledger-compatible assertions. Because
+--    they're what we've always had, and removing them would break some
+--    journals unnecessarily.  Implemented with = syntax.
+--
+-- 2. total assertions. Because otherwise assertions are a bit leaky.
+--    Implemented with == syntax.
+--
+-- 3. subaccount-inclusive assertions. Because that's something folks need.
+--    Not implemented.
+--
+-- 4. flexible assertions allowing custom criteria (perhaps arbitrary
+--    queries). Because power users have diverse needs and want to try out
+--    different schemes (assert cleared balances, assert balance from real or
+--    virtual postings, etc.). Not implemented.
+--
+-- 5. multicommodity assertions, asserting the balance of multiple commodities
+--    at once. Not implemented, requires #934.
+--
 data BalanceAssertion = BalanceAssertion {
-      baamount   :: Amount,             -- ^ the expected value of a particular commodity
-      baexact    :: Bool,               -- ^ whether the assertion is exclusive, and doesn't allow other commodities alongside 'baamount'
-      baposition :: GenericSourcePos
+      baamount    :: Amount,             -- ^ the expected balance in a particular commodity
+      batotal     :: Bool,               -- ^ disallow additional non-asserted commodities ?
+      bainclusive :: Bool,               -- ^ include subaccounts when calculating the actual balance ?
+      baposition  :: GenericSourcePos    -- ^ the assertion's file position, for error reporting
     } deriving (Eq,Typeable,Data,Generic,Show)
 
 instance NFData BalanceAssertion
@@ -256,10 +293,11 @@ data Posting = Posting {
       pcomment          :: Text,              -- ^ this posting's comment lines, as a single non-indented multi-line string
       ptype             :: PostingType,
       ptags             :: [Tag],                   -- ^ tag names and values, extracted from the comment
-      pbalanceassertion :: Maybe BalanceAssertion,  -- ^ optional: the expected balance in this commodity in the account after this posting
+      pbalanceassertion :: Maybe BalanceAssertion,  -- ^ an expected balance in the account after this posting,
+                                                    --   in a single commodity, excluding subaccounts.
       ptransaction      :: Maybe Transaction,       -- ^ this posting's parent transaction (co-recursive types).
                                                     --   Tying this knot gets tedious, Maybe makes it easier/optional.
-      porigin           :: Maybe Posting            -- ^ When this posting has been transformed in some way
+      poriginal         :: Maybe Posting            -- ^ When this posting has been transformed in some way
                                                     --   (eg its amount or price was inferred, or the account name was
                                                     --   changed by a pivot or budget report), this references the original 
                                                     --   untransformed posting (which will have Nothing in this field).
@@ -268,7 +306,7 @@ data Posting = Posting {
 instance NFData Posting
 
 -- The equality test for postings ignores the parent transaction's
--- identity, to avoid recuring ad infinitum.
+-- identity, to avoid recurring ad infinitum.
 -- XXX could check that it's Just or Nothing.
 instance Eq Posting where
     (==) (Posting a1 b1 c1 d1 e1 f1 g1 h1 i1 _ _) (Posting a2 b2 c2 d2 e2 f2 g2 h2 i2 _ _) =  a1==a2 && b1==b2 && c1==c2 && d1==d2 && e1==e2 && f1==f2 && g1==g2 && h1==h2 && i1==i2
@@ -276,17 +314,17 @@ instance Eq Posting where
 -- | Posting's show instance elides the parent transaction so as not to recurse forever.
 instance Show Posting where
   show Posting{..} = "PostingPP {" ++ intercalate ", " [
-     ("pdate="             ++ show (show pdate))
-    ,("pdate2="            ++ show (show pdate2))
-    ,("pstatus="           ++ show (show pstatus))
-    ,("paccount="          ++ show paccount)
-    ,("pamount="           ++ show pamount)
-    ,("pcomment="          ++ show pcomment)
-    ,("ptype="             ++ show ptype)
-    ,("ptags="             ++ show ptags)
-    ,("pbalanceassertion=" ++ show pbalanceassertion)
-    ,("ptransaction="      ++ show (const "<txn>" <$> ptransaction))
-    ,("porigin="           ++ show porigin)
+     "pdate="             ++ show (show pdate)
+    ,"pdate2="            ++ show (show pdate2)
+    ,"pstatus="           ++ show (show pstatus)
+    ,"paccount="          ++ show paccount
+    ,"pamount="           ++ show pamount
+    ,"pcomment="          ++ show pcomment
+    ,"ptype="             ++ show ptype
+    ,"ptags="             ++ show ptags
+    ,"pbalanceassertion=" ++ show pbalanceassertion
+    ,"ptransaction="      ++ show (ptransaction $> "txn")
+    ,"poriginal="         ++ show poriginal
     ] ++ "}"
 
 -- TODO: needs renaming, or removal if no longer needed. See also TextPosition in Hledger.UI.Editor
@@ -302,8 +340,9 @@ instance NFData GenericSourcePos
 --    prevents the constraint ‘(Data p0)’ from being solved.
 --    Probable fix: use a type annotation to specify what ‘p0’ should be.
 data Transaction = Transaction {
-      tindex                   :: Integer,   -- ^ this transaction's 1-based position in the input stream, or 0 when not available
-      tsourcepos               :: GenericSourcePos,
+      tindex                   :: Integer,   -- ^ this transaction's 1-based position in the transaction stream, or 0 when not available
+      tprecedingcomment        :: Text,      -- ^ any comment lines immediately preceding this transaction
+      tsourcepos               :: GenericSourcePos,  -- ^ the file position where the date starts
       tdate                    :: Day,
       tdate2                   :: Maybe Day,
       tstatus                  :: Status,
@@ -311,8 +350,7 @@ data Transaction = Transaction {
       tdescription             :: Text,
       tcomment                 :: Text,      -- ^ this transaction's comment lines, as a single non-indented multi-line string
       ttags                    :: [Tag],     -- ^ tag names and values, extracted from the comment
-      tpostings                :: [Posting], -- ^ this transaction's postings
-      tpreceding_comment_lines :: Text       -- ^ any comment lines immediately preceding this transaction
+      tpostings                :: [Posting]  -- ^ this transaction's postings
     } deriving (Eq,Typeable,Data,Generic,Show)
 
 instance NFData Transaction
@@ -336,7 +374,7 @@ nulltransactionmodifier = TransactionModifier{
 
 -- | A transaction modifier transformation, which adds an extra posting
 -- to the matched posting's transaction.
--- Can be like a regular posting, or the amount can have the amultiplier flag set,
+-- Can be like a regular posting, or the amount can have the aismultiplier flag set,
 -- indicating that it's a multiplier for the matched posting's amount.
 type TMPostingRule = Posting
 
@@ -410,7 +448,7 @@ data Journal = Journal {
   ,jparsetimeclockentries :: [TimeclockEntry]                       -- ^ timeclock sessions which have not been clocked out
   ,jincludefilestack      :: [FilePath]
   -- principal data
-  ,jdeclaredaccounts      :: [AccountName]                          -- ^ Accounts declared by account directives, in parse order (after journal finalisation) 
+  ,jdeclaredaccounts      :: [(AccountName,AccountDeclarationInfo)] -- ^ Accounts declared by account directives, in parse order (after journal finalisation) 
   ,jdeclaredaccounttypes  :: M.Map AccountType [AccountName]        -- ^ Accounts whose type has been declared in account directives (usually 5 top-level accounts) 
   ,jcommodities           :: M.Map CommoditySymbol Commodity        -- ^ commodities and formats declared by commodity directives
   ,jinferredcommodities   :: M.Map CommoditySymbol AmountStyle      -- ^ commodities and formats inferred from journal amounts  TODO misnamed - jusedstyles
@@ -439,18 +477,36 @@ type ParsedJournal = Journal
 -- The --output-format option selects one of these for output.
 type StorageFormat = String
 
--- | An account, with name, balances and links to parent/subaccounts
--- which let you walk up or down the account tree.
+-- | Extra information about an account that can be derived from
+-- its account directive (and the other account directives).
+data AccountDeclarationInfo = AccountDeclarationInfo {
+   adicomment          :: Text   -- ^ any comment lines following an account directive for this account
+  ,aditags             :: [Tag]  -- ^ tags extracted from the account comment, if any
+  ,adideclarationorder :: Int    -- ^ the order in which this account was declared,
+                                 --   relative to other account declarations, during parsing (1..)
+} deriving (Eq,Show,Data,Generic)
+
+instance NFData AccountDeclarationInfo
+
+nullaccountdeclarationinfo = AccountDeclarationInfo {
+   adicomment          = ""
+  ,aditags             = []
+  ,adideclarationorder = 0
+}
+
+-- | An account, with its balances, parent/subaccount relationships, etc.
+-- Only the name is required; the other fields are added when needed.
 data Account = Account {
-  aname                     :: AccountName,   -- ^ this account's full name
-  adeclarationorder         :: Maybe Int  ,   -- ^ the relative position of this account's account directive, if any. Normally a natural number. 
-  aebalance                 :: MixedAmount,   -- ^ this account's balance, excluding subaccounts
-  asubs                     :: [Account],     -- ^ sub-accounts
-  anumpostings              :: Int,           -- ^ number of postings to this account
-  -- derived from the above :
-  aibalance                 :: MixedAmount,   -- ^ this account's balance, including subaccounts
-  aparent                   :: Maybe Account, -- ^ parent account
-  aboring                   :: Bool           -- ^ used in the accounts report to label elidable parents
+   aname                     :: AccountName    -- ^ this account's full name
+  ,adeclarationinfo          :: Maybe AccountDeclarationInfo  -- ^ optional extra info from account directives
+  -- relationships in the tree
+  ,asubs                     :: [Account]      -- ^ this account's sub-accounts
+  ,aparent                   :: Maybe Account  -- ^ parent account
+  ,aboring                   :: Bool           -- ^ used in the accounts report to label elidable parents
+  -- balance information
+  ,anumpostings              :: Int            -- ^ the number of postings to this account
+  ,aebalance                 :: MixedAmount    -- ^ this account's balance, excluding subaccounts
+  ,aibalance                 :: MixedAmount    -- ^ this account's balance, including subaccounts
   } deriving (Typeable, Data, Generic)
 
 -- | Whether an account's balance is normally a positive number (in 
