@@ -33,7 +33,6 @@ module Hledger.Read.Common (
   rejp,
   genericSourcePos,
   journalSourcePos,
-  applyTransactionModifiers,
   parseAndFinaliseJournal,
   parseAndFinaliseJournal',
   setYear,
@@ -42,7 +41,6 @@ module Hledger.Read.Common (
   getDefaultCommodityAndStyle,
   getDefaultAmountStyle,
   getAmountStyle,
-  pushDeclaredAccount,
   addDeclaredAccountType,
   pushParentAccount,
   popParentAccount,
@@ -229,14 +227,6 @@ journalSourcePos p p' = JournalSourcePos (sourceName p) (fromIntegral . unPos $
             | otherwise = unPos $ sourceLine p' -- might be at end of file withat last new-line
 
 
--- | Apply any transaction modifier rules in the journal 
--- (adding automated postings to transactions, eg).
-applyTransactionModifiers :: Journal -> Journal
-applyTransactionModifiers j = j { jtxns = map applyallmodifiers $ jtxns j }
-  where
-    applyallmodifiers = 
-      foldr (flip (.) . transactionModifierToFunction) id (jtxnmodifiers j)
-
 -- | Given a megaparsec ParsedJournal parser, input options, file
 -- path and file content: parse and post-process a Journal, or give an error.
 parseAndFinaliseJournal :: ErroringJournalParser IO ParsedJournal -> InputOpts
@@ -264,14 +254,24 @@ parseAndFinaliseJournal parser iopts f txt = do
         -- time. If we are only running once, we reorder and follow
         -- the options for checking assertions.
         let fj = if auto_ iopts && (not . null . jtxnmodifiers) pj
-                 then applyTransactionModifiers <$>
-                      (journalBalanceTransactions False $
-                       journalReverse $
-                       journalApplyCommodityStyles pj) >>=
-                      (\j -> journalBalanceTransactions (not $ ignore_assertions_ iopts) $
-                             journalAddFile (f, txt) $
-                             journalSetLastReadTime t $
-                             j)
+
+                 -- transaction modifiers are active
+                 then
+                   -- first pass, doing most of the work
+                     (
+                      (journalModifyTransactions <$>) $  -- add auto postings after balancing ? #893b fails
+                      journalBalanceTransactions False $
+                      -- journalModifyTransactions <$>   -- add auto postings before balancing ? probably #893a, #928, #938 fail
+                      journalReverse $
+                      journalAddFile (f, txt) $
+                      journalApplyCommodityStyles pj)
+                   -- second pass, checking balance assertions
+                   >>= (\j ->
+                      journalBalanceTransactions (not $ ignore_assertions_ iopts) $
+                      journalSetLastReadTime t $
+                      j)
+
+                 -- transaction modifiers are not active
                  else journalBalanceTransactions (not $ ignore_assertions_ iopts) $
                       journalReverse $
                       journalAddFile (f, txt) $
@@ -283,6 +283,8 @@ parseAndFinaliseJournal parser iopts f txt = do
             Right j -> return j
             Left e  -> throwError e
 
+-- Like parseAndFinaliseJournal but takes a (non-Erroring) JournalParser.
+-- Used for timeclock/timedot. XXX let them use parseAndFinaliseJournal instead
 parseAndFinaliseJournal' :: JournalParser IO ParsedJournal -> InputOpts
                            -> FilePath -> Text -> ExceptT String IO Journal
 parseAndFinaliseJournal' parser iopts f txt = do
@@ -303,7 +305,7 @@ parseAndFinaliseJournal' parser iopts f txt = do
       -- time. If we are only running once, we reorder and follow the
       -- options for checking assertions.
       let fj = if auto_ iopts && (not . null . jtxnmodifiers) pj
-               then applyTransactionModifiers <$>
+               then journalModifyTransactions <$>
                     (journalBalanceTransactions False $
                      journalReverse $
                      journalApplyCommodityStyles pj) >>=
@@ -352,9 +354,6 @@ getAmountStyle commodity = do
     let effectiveStyle = listToMaybe $ catMaybes [specificStyle, defaultStyle]
     return effectiveStyle
 
-pushDeclaredAccount :: AccountName -> JournalParser m ()
-pushDeclaredAccount acct = modify' (\j -> j{jdeclaredaccounts = acct : jdeclaredaccounts j})
-
 addDeclaredAccountType :: AccountName -> AccountType -> JournalParser m ()
 addDeclaredAccountType acct atype = 
   modify' (\j -> j{jdeclaredaccounttypes = M.insertWith (++) atype [acct] (jdeclaredaccounttypes j)})
@@ -630,7 +629,7 @@ amountwithoutpricep = do
     let numRegion = (offBeforeNum, offAfterNum)
     (q,prec,mdec,mgrps) <- lift $ interpretNumber numRegion suggestedStyle ambiguousRawNum mExponent
     let s = amountstyle{ascommodityside=L, ascommodityspaced=commodityspaced, asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps}
-    return $ Amount c (sign (sign2 q)) NoPrice s mult
+    return $ nullamt{acommodity=c, aquantity=sign (sign2 q), aismultiplier=mult, astyle=s, aprice=NoPrice}
 
   rightornosymbolamountp :: Bool -> (Decimal -> Decimal) -> JournalParser m Amount
   rightornosymbolamountp mult sign = label "amount" $ do
@@ -646,7 +645,7 @@ amountwithoutpricep = do
         suggestedStyle <- getAmountStyle c
         (q,prec,mdec,mgrps) <- lift $ interpretNumber numRegion suggestedStyle ambiguousRawNum mExponent
         let s = amountstyle{ascommodityside=R, ascommodityspaced=commodityspaced, asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps}
-        return $ Amount c (sign q) NoPrice s mult
+        return $ nullamt{acommodity=c, aquantity=sign q, aismultiplier=mult, astyle=s, aprice=NoPrice}
       -- no symbol amount
       Nothing -> do
         suggestedStyle <- getDefaultAmountStyle
@@ -657,7 +656,7 @@ amountwithoutpricep = do
         let (c,s) = case (mult, defcs) of
               (False, Just (defc,defs)) -> (defc, defs{asprecision=max (asprecision defs) prec})
               _ -> ("", amountstyle{asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps})
-        return $ Amount c (sign q) NoPrice s mult
+        return $ nullamt{acommodity=c, aquantity=sign q, aismultiplier=mult, astyle=s, aprice=NoPrice}
 
   -- For reducing code duplication. Doesn't parse anything. Has the type
   -- of a parser only in order to throw parse errors (for convenience).
@@ -721,7 +720,7 @@ priceamountp = option NoPrice $ do
   priceConstructor <- char '@' *> pure TotalPrice <|> pure UnitPrice
 
   lift (skipMany spacenonewline)
-  priceAmount <- amountwithoutpricep <?> "amount (as a price)"
+  priceAmount <- amountwithoutpricep <?> "unpriced amount (specifying a price)"
 
   pure $ priceConstructor priceAmount
 
@@ -729,16 +728,23 @@ balanceassertionp :: JournalParser m BalanceAssertion
 balanceassertionp = do
   sourcepos <- genericSourcePos <$> lift getSourcePos
   char '='
-  exact <- optional $ try $ char '='
+  istotal <- fmap isJust $ optional $ try $ char '='
+  isinclusive <- fmap isJust $ optional $ try $ char '*'
   lift (skipMany spacenonewline)
-  a <- amountp <?> "amount (for a balance assertion or assignment)" -- XXX should restrict to a simple amount
+  -- this amount can have a price; balance assertions ignore it,
+  -- but balance assignments will use it
+  a <- amountp <?> "amount (for a balance assertion or assignment)"
   return BalanceAssertion
-    { baamount = a
-    , baexact = isJust exact
-    , baposition = sourcepos
+    { baamount    = a
+    , batotal     = istotal
+    , bainclusive = isinclusive
+    , baposition  = sourcepos
     }
 
--- http://ledger-cli.org/3.0/doc/ledger3.html#Fixing-Lot-Prices
+-- Parse a Ledger-style fixed lot price: {=PRICE}
+-- https://www.ledger-cli.org/3.0/doc/ledger3.html#Fixing-Lot-Prices .
+-- Currently we ignore these (hledger's @ PRICE is equivalent),
+-- and we don't parse a Ledger-style {PRICE} (equivalent to Ledger's @ PRICE).
 fixedlotpricep :: JournalParser m (Maybe Amount)
 fixedlotpricep = optional $ do
   try $ do
@@ -747,7 +753,7 @@ fixedlotpricep = optional $ do
   lift (skipMany spacenonewline)
   char '='
   lift (skipMany spacenonewline)
-  a <- amountp -- XXX should restrict to a simple amount
+  a <- amountwithoutpricep <?> "unpriced amount (for an ignored ledger-style fixed lot price)"
   lift (skipMany spacenonewline)
   char '}'
   return a
@@ -1297,7 +1303,7 @@ match' p = do
 
 tests_Common = tests "Common" [
 
-  tests "amountp" [
+   tests "amountp" [
     test "basic"                  $ expectParseEq amountp "$47.18"     (usd 47.18)
    ,test "ends with decimal mark" $ expectParseEq amountp "$1."        (usd 1  `withPrecision` 0)
    ,test "unit price"             $ expectParseEq amountp "$10 @ €0.5" 
