@@ -66,9 +66,8 @@ import Prelude ()
 import "base-compat-batteries" Prelude.Compat hiding (readFile)
 import qualified Control.Exception as C
 import Control.Monad
-import Control.Monad.Except (ExceptT(..))
+import Control.Monad.Except (ExceptT(..), runExceptT)
 import Control.Monad.State.Strict
-import Data.Maybe
 import qualified Data.Map.Strict as M
 import Data.Text (Text)
 import Data.String
@@ -253,35 +252,80 @@ orRethrowIOError io msg = do
     Right res -> pure res
     Left errMsg -> fail errMsg
 
+-- Parse an account directive, adding its info to the journal's
+-- list of account declarations.
 accountdirectivep :: JournalParser m ()
 accountdirectivep = do
+  off <- getOffset -- XXX figure out a more precise position later
+
   string "account"
   lift (skipSome spacenonewline)
+
   -- the account name, possibly modified by preceding alias or apply account directives
   acct <- modifiedaccountnamep
-  -- and maybe something else after two or more spaces ?
-  matype :: Maybe AccountType <- lift $ fmap (fromMaybe Nothing) $ optional $ try $ do
+
+  -- maybe an account type code (ALERX) after two or more spaces
+  -- XXX added in 1.11, deprecated in 1.13, remove in 1.14
+  mtypecode :: Maybe Char <- lift $ optional $ try $ do
     skipSome spacenonewline -- at least one more space in addition to the one consumed by modifiedaccountp 
-    choice [
-      -- a numeric account code, as supported in 1.9-1.10 ? currently ignored
-       some digitChar >> return Nothing
-      -- a letter account type code (ALERX), as added in 1.11 ?
-      ,char 'A' >> return (Just Asset) 
-      ,char 'L' >> return (Just Liability) 
-      ,char 'E' >> return (Just Equity) 
-      ,char 'R' >> return (Just Revenue) 
-      ,char 'X' >> return (Just Expense) 
-      ]
-  -- and maybe a comment on this and/or following lines ? (ignore for now)
-  (_cmt, _tags) <- lift transactioncommentp
-  -- and maybe Ledger-style subdirectives ? (ignore)
+    choice $ map char "ALERX"
+
+  -- maybe a comment, on this and/or following lines
+  (cmt, tags) <- lift transactioncommentp
+  
+  -- maybe Ledger-style subdirectives (ignored)
   skipMany indentedlinep
 
+  -- an account type may have been set by account type code or a tag;
+  -- the latter takes precedence
+  let
+    mtypecode' :: Maybe Text = maybe
+      (T.singleton <$> mtypecode)
+      Just
+      $ lookup accountTypeTagName tags
+    metype = parseAccountTypeCode <$> mtypecode'
+
   -- update the journal
-  case matype of
-    Nothing    -> return ()
-    Just atype -> addDeclaredAccountType acct atype
-  pushDeclaredAccount acct
+  addAccountDeclaration (acct, cmt, tags)
+  case metype of
+    Nothing         -> return ()
+    Just (Right t)  -> addDeclaredAccountType acct t
+    Just (Left err) -> customFailure $ parseErrorAt off err
+
+-- The special tag used for declaring account type. XXX change to "class" ?
+accountTypeTagName = "type"
+
+parseAccountTypeCode :: Text -> Either String AccountType
+parseAccountTypeCode s =
+  case T.toLower s of
+    "asset"     -> Right Asset
+    "a"         -> Right Asset
+    "liability" -> Right Liability
+    "l"         -> Right Liability
+    "equity"    -> Right Equity
+    "e"         -> Right Equity
+    "revenue"   -> Right Revenue
+    "r"         -> Right Revenue
+    "expense"   -> Right Expense
+    "x"         -> Right Expense
+    _           -> Left err
+  where
+    err = "invalid account type code "++T.unpack s++", should be one of " ++
+          (intercalate ", " $ ["A","L","E","R","X","ASSET","LIABILITY","EQUITY","REVENUE","EXPENSE"])
+
+-- Add an account declaration to the journal, auto-numbering it.
+addAccountDeclaration :: (AccountName,Text,[Tag]) -> JournalParser m ()
+addAccountDeclaration (a,cmt,tags) =
+  modify' (\j ->
+             let
+               decls = jdeclaredaccounts j
+               d     = (a, nullaccountdeclarationinfo{
+                              adicomment          = cmt
+                             ,aditags             = tags
+                             ,adideclarationorder = length decls + 1
+                             })
+             in
+               j{jdeclaredaccounts = d:decls})
 
 indentedlinep :: JournalParser m String
 indentedlinep = lift (skipSome spacenonewline) >> (rstrip <$> lift restofline)
@@ -525,23 +569,18 @@ periodictransactionp = do
       customFailure $ parseErrorAtRegion offset1 offset2 $
            "remainder of period expression cannot be parsed"
         <> "\nperhaps you need to terminate the period expression with a double space?"
+        <> "\na double space is required between period expression and description/comment"
     pure pexp
 
   -- In periodic transactions, the period expression has an additional constraint:
   case checkPeriodicTransactionStartDate interval span periodtxt of
     Just e -> customFailure $ parseErrorAt off e
     Nothing -> pure ()
-  -- The line can end here, or it can continue with one or more spaces
-  -- and then zero or more of the following fields. A bit awkward.
-  (status, code, description, (comment, tags)) <- lift $
-    (<|>) (eolof >> return (Unmarked, "", "", ("", []))) $ do
-      skipSome spacenonewline
-      s         <- statusp
-      c         <- codep
-      desc      <- T.strip <$> descriptionp
-      (cmt, ts) <- transactioncommentp
-      return (s,c,desc,(cmt,ts))
-
+  
+  status <- lift statusp <?> "cleared status"
+  code <- lift codep <?> "transaction code"
+  description <- lift $ T.strip <$> descriptionp
+  (comment, tags) <- lift transactioncommentp
   -- next lines; use same year determined above
   postings <- postingsp (Just $ first3 $ toGregorian refdate)
 
@@ -573,7 +612,7 @@ transactionp = do
   postings <- postingsp (Just year)
   endpos <- getSourcePos
   let sourcepos = journalSourcePos startpos endpos
-  return $ txnTieKnot $ Transaction 0 sourcepos date edate status code description comment tags postings ""
+  return $ txnTieKnot $ Transaction 0 "" sourcepos date edate status code description comment tags postings
 
 --- ** postings
 
@@ -693,6 +732,19 @@ tests_JournalReader = tests "JournalReader" [
         ,ptcomment     = ""
         }
 
+    ,test "Just date, no description" $ expectParseEq periodictransactionp
+      "~ 2019-01-04\n"
+      nullperiodictransaction {
+         ptperiodexpr  = "2019-01-04"
+        ,ptinterval    = NoInterval
+        ,ptspan        = DateSpan (Just $ fromGregorian 2019 1 4) (Just $ fromGregorian 2019 1 5)
+        ,ptdescription = ""
+        ,ptcomment     = ""
+        }
+
+    ,test "Just date, no description + empty transaction comment" $ expectParse periodictransactionp
+      "~ 2019-01-04\n  ;\n  a  1\n  b\n"
+
     ]
 
   ,tests "postingp" [
@@ -760,7 +812,7 @@ tests_JournalReader = tests "JournalReader" [
         ])
       nulltransaction{
         tsourcepos=JournalSourcePos "" (1,7),  -- XXX why 7 here ?
-        tpreceding_comment_lines="",
+        tprecedingcomment="",
         tdate=fromGregorian 2012 5 14,
         tdate2=Just $ fromGregorian 2012 5 15,
         tstatus=Unmarked,
@@ -800,7 +852,16 @@ tests_JournalReader = tests "JournalReader" [
         ,"  b"
         ," "
         ]
-  
+
+    ,test "transactionp parses an empty transaction comment following whitespace line" $
+      expect $ isRight $ rjp transactionp $ T.unlines
+        ["2012/1/1"
+        ,"  ;"
+        ,"  a  1"
+        ,"  b"
+        ," "
+        ]
+
     ,test "comments everywhere, two postings parsed" $
       expectParseEqOn transactionp 
         (T.unlines
@@ -824,11 +885,16 @@ tests_JournalReader = tests "JournalReader" [
     ]
 
   ,test "accountdirectivep" $ do
-    test "with-comment" $ expectParse accountdirectivep "account a:b  ; a comment\n"
+    test "with-comment"       $ expectParse accountdirectivep "account a:b  ; a comment\n"
     test "does-not-support-!" $ expectParseError accountdirectivep "!account a:b\n" ""
-    test "account-sort-code" $ expectParse accountdirectivep "account a:b  1000\n"
-    test "account-type-code" $ expectParse accountdirectivep "account a:b  A\n"
-    test "account-type-tag" $ expectParse accountdirectivep "account a:b  ; type:asset\n"
+    test "account-type-code"  $ expectParse accountdirectivep "account a:b  A\n"
+    test "account-type-tag"   $ expectParseStateOn accountdirectivep "account a:b  ; type:asset\n"
+      jdeclaredaccounts
+      [("a:b", AccountDeclarationInfo{adicomment          = "type:asset\n"
+                                     ,aditags             = [("type","asset")]
+                                     ,adideclarationorder = 1
+                                     })
+      ]
 
   ,test "commodityconversiondirectivep" $ do
      expectParse commodityconversiondirectivep "C 1h = $50.00\n"
@@ -869,4 +935,12 @@ tests_JournalReader = tests "JournalReader" [
     test "empty file" $ expectParseEqE journalp "" nulljournal
     ]
 
+   -- these are defined here rather than in Common so they can use journalp
+  ,tests "parseAndFinaliseJournal" [
+    test "basic" $ do
+        ej <- io $ runExceptT $ parseAndFinaliseJournal journalp definputopts "" "2019-1-1\n"
+        let Right j = ej
+        expectEqPP [""] $ journalFilePaths j
+   ]
+
   ]
