@@ -64,12 +64,18 @@ type BudgetReportRow = PeriodicReportRow BudgetCell
 -- actual balance changes from the regular transactions,
 -- and compare these to get a 'BudgetReport'.
 -- Unbudgeted accounts may be hidden or renamed (see budgetRollup).
-budgetReport :: ReportOpts -> Bool -> Bool -> DateSpan -> Day -> Journal -> BudgetReport
-budgetReport ropts assrt showunbudgeted reportspan d j =
+budgetReport :: ReportOpts -> Bool -> DateSpan -> Day -> Journal -> BudgetReport
+budgetReport ropts' assrt reportspan d j =
   let
+    -- Budget report demands ALTree mode to ensure subaccounts and subaccount budgets are properly handled
+    -- and that reports with and without --empty make sense when compared side by side
+    ropts = ropts' { accountlistmode_ = ALTree }
+    showunbudgeted = empty_ ropts
     q = queryFromOpts d ropts 
     budgetedaccts = 
       dbg2 "budgetedacctsinperiod" $
+      nub $ 
+      concatMap expandAccountName $
       accountNamesFromPostings $ 
       concatMap tpostings $ 
       concatMap (flip runPeriodicTransaction reportspan) $ 
@@ -77,7 +83,7 @@ budgetReport ropts assrt showunbudgeted reportspan d j =
     actualj = dbg1 "actualj" $ budgetRollUp budgetedaccts showunbudgeted j
     budgetj = dbg1 "budgetj" $ budgetJournal assrt ropts reportspan j
     actualreport@(MultiBalanceReport (actualspans, _, _)) = dbg1 "actualreport" $ multiBalanceReport ropts  q actualj
-    budgetgoalreport@(MultiBalanceReport (_, budgetgoalitems, budgetgoaltotals)) = dbg1 "budgetgoalreport" $ multiBalanceReport ropts q budgetj
+    budgetgoalreport@(MultiBalanceReport (_, budgetgoalitems, budgetgoaltotals)) = dbg1 "budgetgoalreport" $ multiBalanceReport (ropts{empty_=True}) q budgetj
     budgetgoalreport'
       -- If no interval is specified:
       -- budgetgoalreport's span might be shorter actualreport's due to periodic txns; 
@@ -157,7 +163,7 @@ budgetJournal assrt _ropts reportspan j =
 --
 -- 2. subaccounts with no budget goal are merged with their closest parent account
 --    with a budget goal, so that only budgeted accounts are shown. 
---    This can be disabled by --show-unbudgeted.
+--    This can be disabled by --empty.
 --
 budgetRollUp :: [AccountName] -> Bool -> Journal -> Journal
 budgetRollUp budgetedaccts showunbudgeted j = j { jtxns = remapTxn <$> jtxns j }
@@ -165,7 +171,7 @@ budgetRollUp budgetedaccts showunbudgeted j = j { jtxns = remapTxn <$> jtxns j }
     remapTxn = mapPostings (map remapPosting)
       where
         mapPostings f t = txnTieKnot $ t { tpostings = f $ tpostings t }
-        remapPosting p = p { paccount = remapAccount $ paccount p, porigin = Just . fromMaybe p $ porigin p }
+        remapPosting p = p { paccount = remapAccount $ paccount p, poriginal = Just . fromMaybe p $ poriginal p }
           where
             remapAccount a
               | hasbudget         = a
@@ -262,18 +268,24 @@ budgetReportSpan (PeriodicReport (spans, _, _)) = DateSpan (spanStart $ head spa
 
 -- | Render a budget report as plain text suitable for console output.
 budgetReportAsText :: ReportOpts -> BudgetReport -> String
-budgetReportAsText ropts budgetr =
+budgetReportAsText ropts budgetr@(PeriodicReport ( _, rows, _)) =
   printf "Budget performance in %s:\n\n" (showDateSpan $ budgetReportSpan budgetr)
   ++ 
-  tableAsText ropts showcell (budgetReportAsTable ropts budgetr)
+  tableAsText ropts showcell (maybetranspose $ budgetReportAsTable ropts budgetr)
   where
+    actualwidth =
+      maximum [ maybe 0 (length . showMixedAmountOneLineWithoutPrice) amt
+      | (_, _, _, amtandgoals, _, _) <- rows
+      , (amt, _) <- amtandgoals ]
+    budgetwidth =
+      maximum [ maybe 0 (length . showMixedAmountOneLineWithoutPrice) goal
+      | (_, _, _, amtandgoals, _, _) <- rows
+      , (_, goal) <- amtandgoals ]
     -- XXX lay out actual, percentage and/or goal in the single table cell for now, should probably use separate cells
     showcell :: (Maybe Change, Maybe BudgetGoal) -> String
     showcell (mactual, mbudget) = actualstr ++ " " ++ budgetstr
       where
-        actualwidth  = 7
         percentwidth = 4
-        budgetwidth  = 5
         actual = fromMaybe 0 mactual
         actualstr = printf ("%"++show actualwidth++"s") (showamt actual)
         budgetstr = case mbudget of
@@ -308,6 +320,9 @@ budgetReportAsText ropts budgetr =
     -- don't show the budget amount in color, it messes up alignment
     showbudgetamt = showMixedAmountOneLineWithoutPrice
 
+    maybetranspose | transpose_ ropts = \(Table rh ch vals) -> Table ch rh (transpose vals)
+                   | otherwise       = id
+
 -- | Build a 'Table' from a multi-column balance report.
 budgetReportAsTable :: ReportOpts -> BudgetReport -> Table String String (Maybe MixedAmount, Maybe MixedAmount)
 budgetReportAsTable 
